var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { defaultWithdrawalGasLimit } from './config';
import { TornadoFeeOracle } from './feeOracle';
import { GasPriceOracle } from '@tornado/gas-price-oracle';
import { bump } from './utils';
export class TornadoFeeOracleV4 extends TornadoFeeOracle {
    constructor(chainId, rpcUrl, defaultGasPrices) {
        const oracleConfig = {
            chainId,
            defaultRpc: rpcUrl,
            defaultFallbackGasPrices: defaultGasPrices,
        };
        const gasPriceOracle = new GasPriceOracle(oracleConfig);
        super(4, chainId, rpcUrl, gasPriceOracle);
    }
    getGasLimit(tx, type = 'other', bumpPercent = 0) {
        return __awaiter(this, void 0, void 0, function* () {
            if (type === 'user_withdrawal')
                return bump(defaultWithdrawalGasLimit[this.chainId], bumpPercent).toNumber();
            // Need to bump relayer gas limit for transaction, because predefined gas limit to small to be 100% sure that transaction will be sent
            // This leads to fact that relayer often pays extra for gas from his own funds, however, this was designed by previous developers
            if (type === 'relayer_withdrawal')
                return bump(defaultWithdrawalGasLimit[this.chainId], bumpPercent || 25).toNumber();
            // For compatibility reasons, when wee check user-provided fee for V4 withdrawal transaction, we need dump gas limit
            // for about 20 percent,so that the transaction will be sent, even if it results in some loss for the relayer
            if (type === 'relayer_withdrawal_check_v4')
                return bump(defaultWithdrawalGasLimit[this.chainId], -25).toNumber();
            if (!tx || Object.keys(tx).length === 0)
                return bump(23000, bumpPercent).toNumber();
            return bump(yield this.provider.estimateGas(tx), bumpPercent).toNumber();
        });
    }
}
//# sourceMappingURL=feeOracleV4.js.map