var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { ChainId } from './config';
import { TornadoFeeOracle } from './feeOracle';
import { GasPriceOracle } from '@tornado/gas-price-oracle';
import { bump } from './utils';
import { TornadoFeeOracleV4 } from './feeOracleV4';
export class TornadoFeeOracleV5 extends TornadoFeeOracle {
    constructor(chainId, rpcUrl, defaultGasPrices) {
        const oracleConfig = {
            chainId,
            defaultRpc: rpcUrl,
            minPriority: chainId === ChainId.MAINNET || chainId === ChainId.GOERLI ? 2 : chainId === ChainId.BSC ? 3 : 0.05,
            percentile: 5,
            blocksCount: 20,
            defaultFallbackGasPrices: defaultGasPrices,
        };
        const gasPriceOracle = new GasPriceOracle(oracleConfig);
        super(5, chainId, rpcUrl, gasPriceOracle);
        this.fallbackFeeOracle = new TornadoFeeOracleV4(chainId, rpcUrl, defaultGasPrices);
    }
    getGasLimit(tx, type = 'other', bumpPercent = 20) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!tx || Object.keys(tx).length === 0)
                return this.fallbackFeeOracle.getGasLimit(tx, type, bumpPercent);
            /* Relayer gas limit must be lower so that fluctuations in gas price cannot lead to the fact that
             * the relayer will actually pay for gas more than the money allocated for this by the user
             * (that is, in fact, relayer will pay for gas from his own money, unless we make the bump percent less for him)
             */
            if (type === 'relayer_withdrawal')
                bumpPercent = 10;
            if (type === 'user_withdrawal')
                bumpPercent = 30;
            try {
                const fetchedGasLimit = yield this.provider.estimateGas(tx);
                return bump(fetchedGasLimit, bumpPercent).toNumber();
            }
            catch (e) {
                return this.fallbackFeeOracle.getGasLimit(tx, type, bumpPercent);
            }
        });
    }
    getGasPriceParams(speed, bumpPercent) {
        const _super = Object.create(null, {
            getGasPriceParams: { get: () => super.getGasPriceParams }
        });
        return __awaiter(this, void 0, void 0, function* () {
            // Only if bump percent didn't provided (if user provides 0, no need to recalculate)
            if (bumpPercent === undefined) {
                switch (this.chainId) {
                    case ChainId.GOERLI:
                        bumpPercent = 100;
                        break;
                    case ChainId.POLYGON:
                    case ChainId.AVAX:
                    case ChainId.XDAI:
                        bumpPercent = 30;
                        break;
                    default:
                        bumpPercent = 10;
                }
            }
            return _super.getGasPriceParams.call(this, speed, bumpPercent);
        });
    }
}
//# sourceMappingURL=feeOracleV5.js.map