"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (g && (g = 0, op[0] && (_ = 0)), _) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var __spreadArray = (this && this.__spreadArray) || function (to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.TokenPriceOracle = void 0;
var factories_1 = require("./contracts/factories");
var ethers_1 = require("ethers");
var utils_1 = require("ethers/lib/utils");
var tornado_config_1 = require("@tornado/tornado-config");
var config_1 = require("./config");
/**
 * Filter non-native tokens from Tornado instances config
 * @param instances Tornado instances from torn-token config ('@tornado/tornado-config' library)
 * @returns Array of non-native tokens
 */
function filterTokensFromTornadoInstances(instances) {
    var ethInstances = Object.values(instances[config_1.ChainId.MAINNET]);
    return ethInstances.filter(function (tokenInstance) { return !!tokenInstance.tokenAddress; });
}
var tornToken = {
    tokenAddress: '0x77777FeDdddFfC19Ff86DB637967013e6C6A116C',
    symbol: 'torn',
    decimals: 18,
};
var tornadoTokens = __spreadArray([tornToken], filterTokensFromTornadoInstances(tornado_config_1.instances), true);
var defaultTornadoTokenPrices = {
    torn: '1689423546359032',
    dai: '598416104472725',
    cdai: '13384388487019',
    usdc: '599013776676721',
    usdt: '599323410893614',
    wbtc: '15659889148334216720',
};
var TokenPriceOracle = /** @class */ (function () {
    /**
     *
     * @param rpcUrl
     * @param tokens
     */
    function TokenPriceOracle(rpcUrl, tokens, defaultTokenPrices) {
        if (tokens === void 0) { tokens = tornadoTokens; }
        if (defaultTokenPrices === void 0) { defaultTokenPrices = defaultTornadoTokenPrices; }
        this.tokens = tokens;
        this.defaultTokenPrices = defaultTokenPrices;
        this.provider = new ethers_1.ethers.providers.JsonRpcProvider(rpcUrl);
        this.oracle = (0, factories_1.getOffchainOracleContract)(this.provider);
        this.multiCall = (0, factories_1.getMultiCallContract)(this.provider);
    }
    Object.defineProperty(TokenPriceOracle.prototype, "defaultPrices", {
        // Instant return default token prices
        get: function () {
            return this.defaultTokenPrices;
        },
        enumerable: false,
        configurable: true
    });
    /**
     * Prepare data for MultiCall contract
     * @param {Token[]} tokens Tokens array
     * @returns Valid structure to provide to MultiCall contract
     */
    TokenPriceOracle.prototype.prepareCallData = function (tokens) {
        var _this = this;
        if (tokens === void 0) { tokens = this.tokens; }
        return tokens.map(function (token) { return ({
            to: _this.oracle.address,
            data: _this.oracle.interface.encodeFunctionData('getRateToEth', [token.tokenAddress, true]),
        }); });
    };
    /**
     * Fetch actual tokens price rate to ETH from offchain oracles
     * @param {Token[]} tokens Token array
     * @returns {TokenPrices} Object with token price rate to ETH in WEI
     */
    TokenPriceOracle.prototype.fetchPrices = function (tokens) {
        if (tokens === void 0) { tokens = this.tokens; }
        return __awaiter(this, void 0, void 0, function () {
            var callData, _a, results, success, prices, i, tokenSymbol, decodedRate, tokenDecimals, ethDecimals, price, e_1;
            return __generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        _b.trys.push([0, 2, , 3]);
                        if (!(tokens === null || tokens === void 0 ? void 0 : tokens.length))
                            return [2 /*return*/, {}];
                        callData = this.prepareCallData(tokens);
                        return [4 /*yield*/, this.multiCall.multicall(callData)];
                    case 1:
                        _a = _b.sent(), results = _a.results, success = _a.success;
                        prices = {};
                        for (i = 0; i < results.length; i++) {
                            tokenSymbol = tokens[i].symbol.toLowerCase();
                            if (!success[i]) {
                                if (this.defaultTokenPrices[tokenSymbol])
                                    prices[tokenSymbol] = this.defaultTokenPrices[tokenSymbol];
                                continue;
                            }
                            decodedRate = utils_1.defaultAbiCoder.decode(['uint256'], results[i]).toString();
                            tokenDecimals = ethers_1.BigNumber.from(10).pow(tokens[i].decimals);
                            ethDecimals = ethers_1.BigNumber.from(10).pow(18);
                            price = ethers_1.BigNumber.from(decodedRate).mul(tokenDecimals).div(ethDecimals);
                            prices[tokenSymbol] = price.toString();
                        }
                        return [2 /*return*/, prices];
                    case 2:
                        e_1 = _b.sent();
                        console.error('Cannot get token prices, return default: ' + e_1);
                        return [2 /*return*/, this.defaultTokenPrices];
                    case 3: return [2 /*return*/];
                }
            });
        });
    };
    return TokenPriceOracle;
}());
exports.TokenPriceOracle = TokenPriceOracle;
//# sourceMappingURL=tokenPriceOracle.js.map