import { BigNumberish, BytesLike } from 'ethers';
import { GasPriceKey, GetTxGasParamsRes } from '@tornado/gas-price-oracle/lib/services';
import { AvailableTokenSymbols } from '@tornado/tornado-config';
import { InstanceTokenSymbol } from './config';
export type HexadecimalStringifiedNumber = string;
export type LegacyGasPriceKey = GasPriceKey;
export type GasPriceParams = GetTxGasParamsRes;
export type LegacyGasPrices = {
    [gasPriceType in LegacyGasPriceKey]: number;
};
export type WithdrawalTxType = 'user_withdrawal' | 'relayer_withdrawal' | 'relayer_withdrawal_check_v4';
export type TxType = WithdrawalTxType | 'other';
export interface TransactionData {
    to: string;
    from?: string;
    nonce?: number;
    gasLimit?: BigNumberish;
    gasPrice?: BigNumberish;
    data?: string;
    value: BigNumberish;
    chainId?: number;
    type?: number;
    maxFeePerGas?: BigNumberish;
    maxPriorityFeePerGas?: BigNumberish;
}
export interface ITornadoFeeOracle {
    getGasParams: (params?: GetGasParamsInput) => Promise<GetGasParamsRes>;
    getGas: (params?: GetGasInput) => Promise<HexadecimalStringifiedNumber>;
    getGasPriceParams: (speed?: LegacyGasPriceKey, bumpPercent?: number) => Promise<GasPriceParams>;
    getGasPrice: (speed?: LegacyGasPriceKey, bumpPercent?: number) => Promise<HexadecimalStringifiedNumber>;
    getGasLimit: (tx?: TransactionData, type?: TxType, bumpPercent?: number) => Promise<number>;
    fetchL1OptimismFee: (tx?: TransactionData) => Promise<HexadecimalStringifiedNumber>;
    calculateRefundInETH: (gasPrice: BigNumberish, tokenSymbol: InstanceTokenSymbol) => HexadecimalStringifiedNumber;
    fetchRefundInETH: (tokenSymbol: InstanceTokenSymbol) => Promise<HexadecimalStringifiedNumber>;
    calculateRefundInToken: (gasPrice: BigNumberish, tokenPriceInEth: BigNumberish, tokenDecimals: HexadecimalStringifiedNumber | number, tokenSymbol: InstanceTokenSymbol) => HexadecimalStringifiedNumber;
    calculateWithdrawalFeeViaRelayer: (params: GetWithdrawalFeeViaRelayerInput) => Promise<HexadecimalStringifiedNumber>;
}
export interface ITornadoPriceOracle {
    defaultPrices: TokenPrices;
    fetchPrices: (tokens?: Token[]) => Promise<TokenPrices>;
}
export type WithdrawalData = {
    contract: string;
    proof: BytesLike;
    args: [BytesLike, BytesLike, string, string, BigNumberish, BigNumberish];
};
export type TornadoPoolInstance = {
    currency: AvailableTokenSymbols;
    amount: number;
    decimals: number;
};
export type TokenSymbol = 'dai' | 'cdai' | 'usdc' | 'usdt' | 'wbtc' | 'torn';
export type Token = {
    tokenAddress: string;
    symbol: TokenSymbol;
    decimals: number;
};
export type TokenPrices = {
    [tokenSymbol in TokenSymbol]?: BigNumberish;
};
export type GetGasParamsRes = {
    gasLimit: number;
    gasPrice: HexadecimalStringifiedNumber;
};
export type GetGasInput = {
    txType?: TxType;
    tx?: TransactionData;
    predefinedGasPrice?: HexadecimalStringifiedNumber;
    predefinedGasLimit?: number;
    bumpGasLimitPercent?: number;
    bumpGasPricePercent?: number;
    speed?: LegacyGasPriceKey;
};
export type GetGasParamsInput = GetGasInput & {
    includeL1FeeToGasLimit?: boolean;
};
export type GetWithdrawalFeeViaRelayerInput = {
    txType: WithdrawalTxType;
    tx?: TransactionData;
    relayerFeePercent: number | string;
    currency: AvailableTokenSymbols | Uppercase<AvailableTokenSymbols>;
    amount: string | number;
    decimals: string | number;
    refundInEth?: HexadecimalStringifiedNumber;
    tokenPriceInEth?: HexadecimalStringifiedNumber | string;
    predefinedGasPrice?: HexadecimalStringifiedNumber;
    predefinedGasLimit?: number;
};
